package edu.vanderbilt.masi.plugins.labelfusion;

import java.io.FileNotFoundException;
import edu.jhu.ece.iacl.jist.pipeline.AlgorithmInformation;
import edu.jhu.ece.iacl.jist.pipeline.AlgorithmRuntimeException;
import edu.jhu.ece.iacl.jist.pipeline.CalculationMonitor;
import edu.jhu.ece.iacl.jist.pipeline.DevelopmentStatus;
import edu.jhu.ece.iacl.jist.pipeline.AlgorithmInformation.AlgorithmAuthor;
import edu.jhu.ece.iacl.jist.pipeline.ProcessingAlgorithm;
import edu.jhu.ece.iacl.jist.pipeline.parameter.ParamCollection;
import edu.vanderbilt.masi.algorithms.labelfusion.AdaptiveLabelFusion;
import edu.vanderbilt.masi.algorithms.labelfusion.LabelFusionBase;
import edu.vanderbilt.masi.algorithms.labelfusion.ObservationBase;

public class PluginAdaptiveLabelFusion extends AbstractVotingFusionPlugin {
	
	/****************************************************
	 * CVS Version Control
	 ****************************************************/
	private static final String cvsversion = "$Revision: 1.1 $";
	private static final String revnum = cvsversion.replace("Revision: ", "").replace("$", "").replace(" ", "");
	private static final String shortDescription = "Runs the adaptive label fusion algorithm.";
	private static final String longDescription = "";
	
	/*
	 * (non-Javadoc)
	 *
	 * @see edu.jhu.ece.iacl.pipeline.ProcessingAlgorithm#createInputParameters(edu.jhu.ece.iacl.pipeline.parameter.ParamCollection)
	 */
	protected void createInputParameters(ParamCollection inputParams) {
		AlgorithmInformation info = getAlgorithmInformation();
		info.setWebsite("https://masi.vuse.vanderbilt.edu/");
		info.setAffiliation("MASI - Vanderbilt");
		info.add(new AlgorithmAuthor("Andrew Asman","andrew.j.asman@vanderbilt.edu","https://masi.vuse.vanderbilt.edu/index.php/MASI:Andrew_Asman"));
		info.setDescription(shortDescription);
		info.setLongDescription(shortDescription + longDescription);
		info.setVersion(revnum);
		info.setEditable(false);
		info.setStatus(DevelopmentStatus.ALPHA);
		
		inputParams.setPackage("MASI");
		inputParams.setCategory("Label Fusion");
		inputParams.setLabel("Adaptive Label Fusion");
		inputParams.setName("Adaptive_Label_Fusion");
		
		inputParams.add(this.get_main_parameters());
		inputParams.add(this.get_weighting_parameters());
		
		// these aren't mandatory for this plugin
		targetim.setMandatory(false);
		imsvols.setMandatory(false);
	}
	
	/*
	 * (non-Javadoc)
	 *
	 * @see edu.jhu.ece.iacl.pipeline.ProcessingAlgorithm#createOutputParameters(edu.jhu.ece.iacl.pipeline.parameter.ParamCollection)
	 */
	protected void createOutputParameters(ParamCollection outputParams) {
		set_output_parameters();
	}
	
	protected void execute(CalculationMonitor monitor) throws AlgorithmRuntimeException {
		try {
			ExecuteWrapper wrapper=new ExecuteWrapper();
			monitor.observe(wrapper);
			wrapper.execute(this);

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	protected class ExecuteWrapper extends AbstractExecuteWrapper {
		
		public void execute(ProcessingAlgorithm alg) throws FileNotFoundException{

			// get the observation object
			ObservationBase obs = get_observation_type();
			obs.normalize_all();
			
			// run the adaptive label fusion algorithm
			String algname = alg.getAlgorithmName();
			LabelFusionBase fusion = new AdaptiveLabelFusion(obs, String.format("%s_Estimate", algname));
			labelvol.setValue(fusion.run());
							
			// write the label probabilities if desired
			write_label_probabilities(obs, fusion, alg);
		}
	}

}